/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ConfigurationLinkRel, ConfigurationResource, ConfigurationService } from '@admin-client/configuration-shared';
import { ApiRootLinkRel, ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import { BuildInfoComponent } from '@alfa-client/common';
import { HasLinkPipe, createEmptyStateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, existsAsHtmlElement, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ActivatedRoute, Router, RouterOutlet } from '@angular/router';
import { AuthenticationService } from '@authentication';
import {
  AdminLogoIconComponent,
  MailboxIconComponent,
  NavItemComponent,
  NavbarComponent,
  OrgaUnitIconComponent,
  UsersIconComponent,
} from '@ods/system';
import { createConfigurationResource } from 'libs/admin/configuration-shared/test/configuration';
import { MenuContainerComponent } from 'libs/admin/configuration/src/lib/menu-container/menu-container.component';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent, MockDirective } from 'ng-mocks';
import { Subscription, of } from 'rxjs';
import { UserProfileButtonContainerComponent } from '../common/user-profile-button-container/user-profile.button-container.component';
import { UnavailablePageComponent } from '../pages/unavailable/unavailable-page/unavailable-page.component';
import { AppComponent } from './app.component';

describe('AppComponent', () => {
  let component: AppComponent;
  let fixture: ComponentFixture<AppComponent>;

  const adminHeaderSelector: string = getDataTestIdOf('admin-header');
  const buildInfoSelector: string = getDataTestIdOf('build-info');
  const userProfileButtonSelector: string = getDataTestIdOf('user-profile-button');
  const usersRolesNavigationSelector: string = getDataTestIdOf('users-roles-navigation');

  const organisationsEinheitenNavigationSelector: string = getDataTestIdOf('organisations-einheiten-navigation');
  const routerOutletSelector: string = getDataTestIdOf('router-outlet');
  const menuContainer: string = getDataTestIdOf('menu-container');

  const authenticationService: Mock<AuthenticationService> = {
    ...mock(AuthenticationService),
    login: jest.fn().mockResolvedValue(Promise.resolve()),
  };

  const router: Mock<Router> = mock(Router);
  const route: Mock<ActivatedRoute> = {
    ...mock(ActivatedRoute),
    snapshot: {
      queryParams: {
        iss: 'some-iss',
        state: 'some-state',
        session_state: 'some-session-state',
        code: 'some-code',
      },
    } as any,
  };

  const apiRootService: Mock<ApiRootService> = mock(ApiRootService);
  let configurationService: Mock<ConfigurationService>;

  beforeEach(async () => {
    configurationService = mock(ConfigurationService);

    await TestBed.configureTestingModule({
      imports: [HasLinkPipe],
      declarations: [
        AppComponent,
        MockComponent(AdminLogoIconComponent),
        MockComponent(OrgaUnitIconComponent),
        MockComponent(UsersIconComponent),
        MockComponent(MailboxIconComponent),
        MockComponent(UserProfileButtonContainerComponent),
        MockComponent(UnavailablePageComponent),
        MockComponent(NavbarComponent),
        MockComponent(NavItemComponent),
        MockComponent(BuildInfoComponent),
        MockComponent(MenuContainerComponent),
        MockDirective(RouterOutlet),
      ],
      providers: [
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
        {
          provide: ApiRootService,
          useValue: apiRootService,
        },
        {
          provide: ConfigurationService,
          useValue: configurationService,
        },
        {
          provide: Router,
          useValue: router,
        },
        {
          provide: ActivatedRoute,
          useValue: route,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(AppComponent);
    component = fixture.componentInstance;
  });

  describe('component', () => {
    it(`should have as title 'admin'`, () => {
      const appTitle: string = fixture.componentInstance.title;

      expect(appTitle).toEqual('admin');
    });

    describe('ngOnInit', () => {
      it('should call authService login', () => {
        component.ngOnInit();

        expect(authenticationService.login).toHaveBeenCalled();
      });

      it('should call doAfterLoggedIn', async () => {
        component.doAfterLoggedIn = jest.fn();

        component.ngOnInit();
        await fixture.whenStable();

        expect(component.doAfterLoggedIn).toHaveBeenCalled();
      });
    });

    describe('do after logged in', () => {
      beforeEach(() => {
        component.evaluateInitialNavigation = jest.fn();
      });

      it('should call apiRootService to getApiRoot', () => {
        component.doAfterLoggedIn();

        expect(apiRootService.getApiRoot).toHaveBeenCalled();
      });

      it('should call evaluateInitialNavigation', () => {
        component.evaluateInitialNavigation = jest.fn();

        component.doAfterLoggedIn();

        expect(component.evaluateInitialNavigation).toHaveBeenCalled();
      });
    });

    describe('evaluate initial navigation', () => {
      beforeEach(() => {
        component.evaluateNavigationByApiRoot = jest.fn();
      });

      it('should call evaluate navigation by apiRoot', () => {
        const apiRootResource: ApiRootResource = createApiRootResource();
        component.apiRootStateResource$ = of(createStateResource(apiRootResource));

        component.evaluateInitialNavigation();

        expect(component.evaluateNavigationByApiRoot).toHaveBeenCalledWith(apiRootResource);
      });

      it('should NOT call evaluate navigation by apiRoot if resource is loading', () => {
        component.apiRootStateResource$ = of(createEmptyStateResource<ApiRootResource>(true));
        component.evaluateNavigationByApiRoot = jest.fn();

        component.evaluateInitialNavigation();

        expect(component.evaluateNavigationByApiRoot).not.toHaveBeenCalled();
      });
    });

    describe('evaluate navigation api root', () => {
      it('should evaluate navigation by configuration if link exists', () => {
        component.evaluateNavigationByConfiguration = jest.fn();
        const apiRootResource: ApiRootResource = createApiRootResource([ApiRootLinkRel.CONFIGURATION]);

        component.evaluateNavigationByApiRoot(apiRootResource);

        expect(component.evaluateNavigationByConfiguration).toHaveBeenCalled();
      });

      it('should navigate by api root if link is missing', () => {
        component.navigateByApiRoot = jest.fn();
        const apiRootResource: ApiRootResource = createApiRootResource();

        component.evaluateNavigationByApiRoot(apiRootResource);

        expect(component.navigateByApiRoot).toHaveBeenCalledWith(apiRootResource);
      });
    });

    describe('evaluate navigation by configuration', () => {
      const configurationResource: ConfigurationResource = createConfigurationResource();

      beforeEach(() => {
        configurationService.get.mockReturnValue(of(createStateResource(configurationResource)));
        component.navigateByConfiguration = jest.fn();
      });

      it('should call configuration service to get resource', () => {
        component.evaluateNavigationByConfiguration();

        expect(configurationService.get).toHaveBeenCalled();
      });

      it('should call navigate by configuration', () => {
        component.evaluateNavigationByConfiguration();

        expect(component.navigateByConfiguration).toHaveBeenCalledWith(configurationResource);
      });

      it('should NOT call navigate by configuration if resource is loading', () => {
        configurationService.get.mockReturnValue(of(createEmptyStateResource<ConfigurationResource>(true)));

        component.evaluateNavigationByConfiguration();

        expect(component.navigateByConfiguration).not.toHaveBeenCalled();
      });
    });

    describe('navigate by configuration', () => {
      beforeEach(() => {
        component.unsubscribe = jest.fn();
      });

      it('should navigate to postfach if settings link exists', () => {
        component.navigateByConfiguration(createConfigurationResource([ConfigurationLinkRel.SETTING]));

        expect(router.navigate).toHaveBeenCalledWith(['/postfach']);
      });

      it('should navigate to statistik if aggregation mapping link exists', () => {
        component.navigateByConfiguration(createConfigurationResource([ConfigurationLinkRel.AGGREGATION_MAPPINGS]));

        expect(router.navigate).toHaveBeenCalledWith(['/statistik']);
      });

      it('should navigate to unavailable page if no link exists', () => {
        component.navigateByConfiguration(createConfigurationResource());

        expect(router.navigate).toHaveBeenCalledWith(['/unavailable']);
      });

      it('should call unsubscribe', () => {
        component.navigateByConfiguration(createConfigurationResource());

        expect(component.unsubscribe).toHaveBeenCalled();
      });
    });

    describe('navigate by api root', () => {
      beforeEach(() => {
        component.unsubscribe = jest.fn();
      });

      it('should navigate to benutzer und rollen if users link exists', () => {
        const apiRootResource: ApiRootResource = createApiRootResource([ApiRootLinkRel.USERS]);

        component.navigateByApiRoot(apiRootResource);

        expect(router.navigate).toHaveBeenCalledWith(['/benutzer_und_rollen']);
      });

      it('should navigate to unavailable page if no link exists', () => {
        component.navigateByApiRoot(createApiRootResource());

        expect(router.navigate).toHaveBeenCalledWith(['/unavailable']);
      });

      it('should call unsubscribe', () => {
        component.navigateByApiRoot(createApiRootResource());

        expect(component.unsubscribe).toHaveBeenCalled();
      });
    });

    describe('unsubscribe', () => {
      describe('apiRoot subscription', () => {
        it('should unsubscribe if exists', () => {
          component.apiRootSubscription = <any>mock(Subscription);
          component.apiRootSubscription.unsubscribe = jest.fn();

          component.unsubscribe();

          expect(component.apiRootSubscription.unsubscribe).toHaveBeenCalled();
        });

        it('should do nothing if not exists', () => {
          component.apiRootSubscription = undefined;
          const unsubscribeSpy: jest.SpyInstance = jest.spyOn(Subscription.prototype, 'unsubscribe');

          component.unsubscribe();

          expect(unsubscribeSpy).not.toHaveBeenCalled();
          unsubscribeSpy.mockRestore();
        });
      });

      describe('configuration subscription', () => {
        it('should unsubscribe if exists', () => {
          component.configurationSubscription = <any>mock(Subscription);
          component.configurationSubscription.unsubscribe = jest.fn();

          component.unsubscribe();

          expect(component.configurationSubscription.unsubscribe).toHaveBeenCalled();
        });

        it('should do nothing if not exists', () => {
          component.apiRootSubscription = undefined;
          const unsubscribeSpy: jest.SpyInstance = jest.spyOn(Subscription.prototype, 'unsubscribe');

          component.unsubscribe();

          expect(unsubscribeSpy).not.toHaveBeenCalled();
          unsubscribeSpy.mockRestore();
        });
      });
    });
  });

  describe('template', () => {
    it('show not show header if apiRoot is not loaded', () => {
      component.apiRootStateResource$ = of(createEmptyStateResource<ApiRootResource>());

      notExistsAsHtmlElement(fixture, adminHeaderSelector);
    });

    describe('user profile button', () => {
      beforeEach(() => {
        component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));
      });

      it('should show if apiRoot exists', () => {
        fixture.detectChanges();

        existsAsHtmlElement(fixture, userProfileButtonSelector);
      });
    });

    describe('navigation', () => {
      describe('user and roles', () => {
        it('should show if users link is present', () => {
          component.apiRootStateResource$ = of(
            createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION, ApiRootLinkRel.USERS])),
          );

          fixture.detectChanges();

          existsAsHtmlElement(fixture, usersRolesNavigationSelector);
        });

        it('should hide if link is missing', () => {
          component.apiRootStateResource$ = of(createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION])));

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, usersRolesNavigationSelector);
        });
      });

      describe('organisationEinheiten', () => {
        it('should show if link in apiRoot exists', () => {
          component.apiRootStateResource$ = of(
            createStateResource(createApiRootResource([ApiRootLinkRel.ORGANISATIONS_EINHEIT, ApiRootLinkRel.CONFIGURATION])),
          );
          fixture.detectChanges();

          existsAsHtmlElement(fixture, organisationsEinheitenNavigationSelector);
        });

        it('should hide if link in apiRoot does not exist', () => {
          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, organisationsEinheitenNavigationSelector);
        });
      });

      describe('menu', () => {
        it('should show if link exists', () => {
          component.apiRootStateResource$ = of(createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION])));

          fixture.detectChanges();

          existsAsHtmlElement(fixture, menuContainer);
        });

        it('should hide if link is missing', () => {
          component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));

          fixture.detectChanges();

          notExistsAsHtmlElement(fixture, menuContainer);
        });
      });
    });

    describe('build version', () => {
      it('should not be rendered if api root not loaded', () => {
        notExistsAsHtmlElement(fixture, buildInfoSelector);
      });

      it('should show after apiRoot loaded', () => {
        component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));
        fixture.detectChanges();

        existsAsHtmlElement(fixture, buildInfoSelector);
      });
    });

    describe('router outlet', () => {
      beforeEach(() => {});

      it('should exist if configuration resource available', () => {
        component.apiRootStateResource$ = of(createStateResource(createApiRootResource([ApiRootLinkRel.CONFIGURATION])));

        fixture.detectChanges();

        existsAsHtmlElement(fixture, routerOutletSelector);
      });

      it('should not exist if configuration resource not available', () => {
        component.apiRootStateResource$ = of(createStateResource(createApiRootResource()));

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, routerOutletSelector);
      });
    });
  });
});
