/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ConfigurationResource, ConfigurationService } from '@admin-client/configuration-shared';
import { ApiRootResource, ApiRootService } from '@alfa-client/api-root-shared';
import { createStateResource, LinkRelationName, StateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { ActivatedRouteSnapshot, Router, UrlTree } from '@angular/router';
import { Resource } from '@ngxp/rest';
import { createConfigurationResource } from 'libs/admin/configuration-shared/test/configuration';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { DummyLinkRel } from 'libs/tech-shared/test/dummy';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { Observable, of } from 'rxjs';
import { GuardData } from './app.routes';

import * as Guard from './app.guard';

describe('Guard', () => {
  const linkRelName: LinkRelationName = DummyLinkRel.DUMMY;
  const route: ActivatedRouteSnapshot = <any>{ data: <GuardData>{ linkRelName } };

  describe('api root guard', () => {
    let apiRootService: Mock<ApiRootService>;

    const apiRootStateResource$: Observable<StateResource<ApiRootResource>> = of(createStateResource(createApiRootResource()));

    beforeEach(() => {
      apiRootService = mock(ApiRootService);

      TestBed.configureTestingModule({
        providers: [
          {
            provide: ApiRootService,
            useValue: apiRootService,
          },
        ],
      });
    });

    beforeEach(() => {
      apiRootService.getApiRoot.mockReturnValue(apiRootStateResource$);
      jest.spyOn(Guard, 'evaluate').mockReturnValue(of(true));
    });

    afterEach(() => {
      jest.restoreAllMocks();
    });

    it('should call apiRootService', () => {
      runGuard().subscribe();

      expect(apiRootService.getApiRoot).toHaveBeenCalled();
    });

    it('should call evaluate', () => {
      const evaluateSpy: jest.SpyInstance = jest.spyOn(Guard, 'evaluate').mockReturnValue(of(true));

      runGuard().subscribe();

      expect(evaluateSpy).toHaveBeenCalledWith(apiRootStateResource$, DummyLinkRel.DUMMY);
    });

    function runGuard(): Observable<boolean | UrlTree> {
      return <Observable<boolean | UrlTree>>TestBed.runInInjectionContext(() => Guard.apiRootGuard(route, null));
    }
  });

  describe('configuration guard', () => {
    let configurationService: Mock<ConfigurationService>;

    const configurationStateResource$: Observable<StateResource<ConfigurationResource>> = of(
      createStateResource(createConfigurationResource()),
    );

    beforeEach(() => {
      configurationService = mock(ConfigurationService);

      TestBed.configureTestingModule({
        providers: [
          {
            provide: ConfigurationService,
            useValue: configurationService,
          },
        ],
      });
    });

    beforeEach(() => {
      configurationService.get.mockReturnValue(configurationStateResource$);
      jest.spyOn(Guard, 'evaluate').mockReturnValue(of(true));
    });

    afterEach(() => {
      jest.restoreAllMocks();
    });

    it('should call configurationService', () => {
      runGuard().subscribe();

      expect(configurationService.get).toHaveBeenCalled();
    });

    it('should call evaluate', () => {
      const evaluateSpy: jest.SpyInstance = jest.spyOn(Guard, 'evaluate').mockReturnValue(of(true));

      runGuard().subscribe();

      expect(evaluateSpy).toHaveBeenCalledWith(configurationStateResource$, DummyLinkRel.DUMMY);
    });

    function runGuard(): Observable<boolean | UrlTree> {
      return <Observable<boolean | UrlTree>>TestBed.runInInjectionContext(() => Guard.configurationGuard(route, null));
    }
  });

  describe('evaluate', () => {
    const resource: Resource = createDummyResource([DummyLinkRel.DUMMY]);
    const stateResource$: Observable<StateResource<Resource>> = of(createStateResource(resource));

    afterEach(() => {
      jest.restoreAllMocks();
    });

    it('should evaluate resource', () => {
      const urlTreeMock: Mock<UrlTree> = mock(UrlTree);
      const evaluateResourceSpy: jest.SpyInstance = jest
        .spyOn(Guard, 'evaluateResource')
        .mockReturnValue(useFromMock(urlTreeMock));

      evaluate().subscribe();

      expect(evaluateResourceSpy).toHaveBeenCalledWith(resource, DummyLinkRel.DUMMY);
    });

    it('should return evaluated boolean', (done) => {
      jest.spyOn(Guard, 'evaluateResource').mockReturnValue(true);

      evaluate().subscribe((resolvedValue) => {
        expect(resolvedValue).toEqual(true);
        done();
      });
    });

    it('should return evaluated url tree', (done) => {
      const urlTreeMock: Mock<UrlTree> = mock(UrlTree);
      jest.spyOn(Guard, 'evaluateResource').mockReturnValue(useFromMock(urlTreeMock));

      evaluate().subscribe((resolvedValue) => {
        expect(resolvedValue).toEqual(urlTreeMock);
        done();
      });
    });

    function evaluate(): Observable<boolean | UrlTree> {
      return <Observable<boolean | UrlTree>>(
        TestBed.runInInjectionContext(() => Guard.evaluate(stateResource$, DummyLinkRel.DUMMY))
      );
    }
  });

  describe('evaluate resource', () => {
    let router: Mock<Router>;

    beforeEach(() => {
      router = mock(Router);

      TestBed.configureTestingModule({
        providers: [
          {
            provide: Router,
            useValue: router,
          },
        ],
      });
    });

    afterEach(() => {
      jest.restoreAllMocks();
    });

    it('should return true if link exists', () => {
      const result: boolean = <boolean>evaluateResource(createDummyResource([DummyLinkRel.DUMMY]));

      expect(result).toBeTruthy();
    });

    describe('if link not exists', () => {
      it('should call router', () => {
        evaluateResource(createDummyResource());

        expect(router.createUrlTree).toHaveBeenCalledWith(['/unavailable']);
      });

      it('should return redirect route', () => {
        const urlTree: Mock<UrlTree> = mock(UrlTree);
        router.createUrlTree.mockReturnValue(urlTree);

        const result: UrlTree = <UrlTree>evaluateResource(createDummyResource());

        expect(result).toEqual(urlTree);
      });
    });

    function evaluateResource(dummyResource: Resource): boolean | UrlTree {
      return <boolean | UrlTree>TestBed.runInInjectionContext(() => Guard.evaluateResource(dummyResource, DummyLinkRel.DUMMY));
    }
  });
});
